/*  Sirikata
 *  OSegTestMotionPath.cpp
 *
 *  Copyright (c) 2010, Daniel Reiter Horn
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of Sirikata nor the names of its contributors may
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "OSegTestMotionPath.hpp"
#include <sirikata/core/util/Random.hpp>

#include <algorithm>
#include <cmath>


namespace Sirikata {


OSegTestMotionPath::OSegTestMotionPath(const Time& start, const Time& end, const Vector3f& startpos, float32 speed, const Duration& update_period, const BoundingBox3f& region, float zfactor, Vector3f driftDir)
  //OSegTestMotionPath::OSegTestMotionPath(const Time& start, const Time& end, const Vector3f& startpos, float32 speed, const Duration& update_period, const BoundingBox3f& region, float zfactor)
{
  TimedMotionVector3f last_motion(start, MotionVector3f(startpos, Vector3f(0,0,0)));
  mUpdates.push_back(last_motion);

  Time offset_start = start + update_period * randFloat();

  for(Time curtime = offset_start; curtime < end; curtime += update_period)
  {
    Vector3f curpos = last_motion.extrapolate(curtime).position();

    //    Vector3f dir = UniformSampleSphere( randFloat(), randFloat() ) * speed;
    //    dir.z = dir.z * zfactor;

    //    Vector3f dir = Vector3f(.3,.3,.3);
    //    dir = dir.normal() *  speed;
    //    last_motion = TimedMotionVector3f(curtime, MotionVector3f(curpos, dir));

    Vector3f dir = driftDir;
    //dir = dir.normal() *  speed;
    last_motion = TimedMotionVector3f(curtime, MotionVector3f(curpos, dir));

    // last_motion is what we would like, no we have to make sure we'll stay  in range
    Vector3f nextpos = last_motion.extrapolate(curtime + update_period).position();
    Vector3f nextpos_clamped = region.clamp(nextpos);
    Vector3f to_dest = nextpos_clamped - curpos;
    dir = to_dest * (1.f / update_period.toSeconds());

    last_motion = TimedMotionVector3f(curtime, MotionVector3f(curpos, dir));

    mUpdates.push_back(last_motion);
  }
}


const TimedMotionVector3f OSegTestMotionPath::initial() const {
    assert( !mUpdates.empty() );
    return mUpdates[0];
}


const TimedMotionVector3f* OSegTestMotionPath::nextUpdate(const Time& curtime) const {
    for(uint32 i = 0; i < mUpdates.size(); i++)
        if (mUpdates[i].time() > curtime) return &mUpdates[i];
    return NULL;
}


const TimedMotionVector3f OSegTestMotionPath::at(const Time& t) const {
    for(uint32 i = 1; i < mUpdates.size(); i++)
        if (mUpdates[i].time() > t) return mUpdates[i-1];
    return mUpdates[ mUpdates.size()-1 ];
}


} // namespace Sirikata
